"""
Author: Michel Peltriaux
Organization: Struktur- und Genehmigungsdirektion Nord, Rhineland-Palatinate, Germany
Contact: michel.peltriaux@sgdnord.rlp.de
Created on: 29.07.21

"""
import django_filters
from django.utils.translation import gettext_lazy as _
from django import forms
from django.db.models import QuerySet

from konova.filters import QueryTableFilter, CheckboxTableFilter


class CheckboxCompensationTableFilter(CheckboxTableFilter):
    """ Specialization of regular CheckboxTableFilter for compensation model

    """

    def filter_show_all(self, queryset, name, value) -> QuerySet:
        """ Filters queryset depending on value of 'show_all' setting

        Args:
            queryset ():
            name ():
            value ():

        Returns:

        """
        if not value:
            return queryset.filter(
                intervention__users__in=[self.user],  # requesting user has access
            )
        else:
            return queryset

    def filter_show_recorded(self, queryset, name, value) -> QuerySet:
        """ Filters queryset depending on value of 'show_recorded' setting

        Args:
            queryset ():
            name ():
            value ():

        Returns:

        """
        if not value:
            return queryset.filter(
                intervention__recorded=None,
            )
        else:
            return queryset


class CompensationTableFilter:
    """ TableFilter for compensations

    Based widely on InterventionTableFilter.
    Just some minor changes for Compensation model.

    """
    query_filter = None
    checkbox_filter = None
    qs = None

    def __init__(self, *args, **kwargs):
        user = kwargs.get("user", None)
        qs = kwargs.get("queryset", None)
        request_data = kwargs.get("data", None)

        self.query_filter = QueryTableFilter(
            user=user,
            data=request_data,
            queryset=qs,
        )
        self.checkbox_filter = CheckboxCompensationTableFilter(
            user=user,
            data=request_data,
            queryset=self.query_filter.qs,
        )
        self.qs = self.checkbox_filter.qs


class CheckboxEcoAccountTableFilter(CheckboxTableFilter):
    sr = django_filters.BooleanFilter(
        method='filter_only_show_unrecorded',
        label=_("Show only unrecorded"),
        label_suffix=_(""),
        widget=forms.CheckboxInput()
    )

    def filter_show_all(self, queryset, name, value) -> QuerySet:
        """ Filters queryset depending on value of 'show_all' setting

        Args:
            queryset ():
            name ():
            value ():

        Returns:

        """
        if not value:
            return queryset.filter(
                users__in=[self.user],  # requesting user has access
            )
        else:
            return queryset

    def filter_only_show_unrecorded(self, queryset, name, value) -> QuerySet:
        """ Filters queryset depending on value of 'show_recorded' setting

        Args:
            queryset ():
            name ():
            value ():

        Returns:

        """
        if value:
            return queryset.filter(
                recorded=None,
            )
        else:
            return queryset


class EcoAccountTableFilter:
    """ TableFilter for eco accounts

    """
    query_filter = None
    checkbox_filter = None
    qs = None

    def __init__(self, *args, **kwargs):
        user = kwargs.get("user", None)
        qs = kwargs.get("queryset", None)
        request_data = kwargs.get("data", None)

        self.query_filter = QueryTableFilter(
            user=user,
            data=request_data,
            queryset=qs,
        )
        self.checkbox_filter = CheckboxEcoAccountTableFilter(
            user=user,
            data=request_data,
            queryset=self.query_filter.qs,
        )
        self.qs = self.checkbox_filter.qs
